@@ -60,7 +60,7 @@ elif host == 'applejack': #Nijmegen
         'database': 'clamopener',
         'table': 'clamusers_clamusers'
     }
-    DEBUG = True
+    DEBUG = False
     REALM = "WEBSERVICES-LST"
     DIGESTOPAQUE = open(environ['CLAM_DIGESTOPAQUEFILE']).read().strip()
 elif host == 'echo' or host == 'nomia' or host == 'echo.uvt.nl' or host == 'nomia.uvt.nl': #Tilburg
